@@ -1,8 +1,9 @@
 package t::CMAETest::ConnectFail;
 use strict;
 use Test::More;
-use t::Cache::Memcached::AnyEvent::Test;
+use t::Util;
 
+sub should_run { 1 }
 sub run {
     my ($pkg, $protocol, $selector) = @_;
     my $memd = test_client(protocol_class => $protocol, selector_class => $selector);
@@ -10,7 +11,7 @@ sub run {
     my $bogus_server = 'you.should.not.be.able.to.connect.to.me:11211';
     $memd->add_server( $bogus_server );
 
-    my $key_base = "BigF*ckingTruckHitMe";
+    my $key_base = random_key();
     my $value = join('.', time(), $$, {}, rand());
 
     my $cv = AE::cv;
@@ -37,6 +38,8 @@ sub run {
     $cv->recv;
 
     ok $warn_called, "warn properly called";
+
+    $memd->disconnect;
     done_testing;
 }
 
